"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isObject = exports.FlagCache = exports.validateFlags = void 0;
function validateFlags(flagsInput) {
    if (!isObject(flagsInput)) {
        return;
    }
    const flags = {};
    for (const key in flagsInput) {
        const flag = flagsInput[key];
        if (!isObject(flag))
            return;
        if (typeof flag.value !== "boolean" || typeof flag.key !== "string") {
            return;
        }
        flags[key] = { value: flag.value, key: flag.key };
    }
    return flags;
}
exports.validateFlags = validateFlags;
class FlagCache {
    constructor(storage, staleTimeMs, expireTimeMs) {
        this.storage = storage;
        this.staleTimeMs = staleTimeMs;
        this.expireTimeMs = expireTimeMs;
    }
    set(key, { success, flags, attemptCount, }) {
        var _a;
        let cacheData = {};
        try {
            const cachedResponseRaw = this.storage.get();
            if (cachedResponseRaw) {
                cacheData = (_a = validateCacheData(JSON.parse(cachedResponseRaw))) !== null && _a !== void 0 ? _a : {};
            }
        }
        catch (e) {
            // ignore errors
        }
        cacheData[key] = {
            expireAt: Date.now() + this.expireTimeMs,
            staleAt: Date.now() + this.staleTimeMs,
            flags,
            success,
            attemptCount,
        };
        cacheData = Object.fromEntries(Object.entries(cacheData).filter(([_k, v]) => v.expireAt > Date.now()));
        this.storage.set(JSON.stringify(cacheData));
        return cacheData;
    }
    get(key) {
        try {
            const cachedResponseRaw = this.storage.get();
            if (cachedResponseRaw) {
                const cachedResponse = validateCacheData(JSON.parse(cachedResponseRaw));
                if (cachedResponse &&
                    cachedResponse[key] &&
                    cachedResponse[key].expireAt > Date.now()) {
                    return {
                        flags: cachedResponse[key].flags,
                        success: cachedResponse[key].success,
                        stale: cachedResponse[key].staleAt < Date.now(),
                        attemptCount: cachedResponse[key].attemptCount,
                    };
                }
            }
        }
        catch (e) {
            console.debug("error loading feature flag cache:", e);
        }
        return;
    }
    clear() {
        this.storage.clear();
    }
}
exports.FlagCache = FlagCache;
function validateCacheData(cacheDataInput) {
    if (!isObject(cacheDataInput)) {
        return;
    }
    const cacheData = {};
    for (const key in cacheDataInput) {
        const cacheEntry = cacheDataInput[key];
        if (!isObject(cacheEntry))
            return;
        if (typeof cacheEntry.expireAt !== "number" ||
            typeof cacheEntry.staleAt !== "number" ||
            typeof cacheEntry.success !== "boolean" ||
            typeof cacheEntry.attemptCount !== "number" ||
            (cacheEntry.flags && !validateFlags(cacheEntry.flags))) {
            return;
        }
        cacheData[key] = {
            expireAt: cacheEntry.expireAt,
            staleAt: cacheEntry.staleAt,
            success: cacheEntry.success,
            flags: cacheEntry.flags,
            attemptCount: cacheEntry.attemptCount,
        };
    }
    return cacheData;
}
// Simple object check.
function isObject(item) {
    return item && typeof item === "object" && !Array.isArray(item);
}
exports.isObject = isObject;
//# sourceMappingURL=flags-cache.js.map