"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFlags = exports.clearCache = exports.FLAGS_EXPIRE_MS = exports.FLAGS_STALE_MS = exports.fetchFlags = exports.mergeDeep = void 0;
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const config_1 = require("./config");
const flags_cache_1 = require("./flags-cache");
// Deep merge two objects.
function mergeDeep(target, ...sources) {
    if (!sources.length)
        return target;
    const source = sources.shift();
    if ((0, flags_cache_1.isObject)(target) && (0, flags_cache_1.isObject)(source)) {
        for (const key in source) {
            if ((0, flags_cache_1.isObject)(source[key])) {
                if (!target[key])
                    Object.assign(target, { [key]: {} });
                mergeDeep(target[key], source[key]);
            }
            else {
                Object.assign(target, { [key]: source[key] });
            }
        }
    }
    return mergeDeep(target, ...sources);
}
exports.mergeDeep = mergeDeep;
function validateFeatureFlagsResponse(response) {
    if (!(0, flags_cache_1.isObject)(response)) {
        return;
    }
    if (typeof response.success !== "boolean" || !(0, flags_cache_1.isObject)(response.flags)) {
        return;
    }
    const flags = (0, flags_cache_1.validateFlags)(response.flags);
    if (!flags) {
        return;
    }
    return {
        success: response.success,
        flags,
    };
}
function flattenJSON(obj) {
    const result = {};
    for (const key in obj) {
        if (typeof obj[key] === "object") {
            const flat = flattenJSON(obj[key]);
            for (const flatKey in flat) {
                result[`${key}.${flatKey}`] = flat[flatKey];
            }
        }
        else {
            result[key] = obj[key];
        }
    }
    return result;
}
const dedupeFetch = {};
function fetchFlags(url, timeoutMs) {
    return __awaiter(this, void 0, void 0, function* () {
        if (url in dedupeFetch) {
            return dedupeFetch[url];
        }
        const fetchFlagsInner = () => __awaiter(this, void 0, void 0, function* () {
            let flags;
            let success = false;
            try {
                const controller = new AbortController();
                const id = setTimeout(() => controller.abort(), timeoutMs);
                // add SDK version to the query params
                const urlObj = new URL(url);
                urlObj.searchParams.append(config_1.SDK_VERSION_HEADER_NAME, config_1.SDK_VERSION);
                const res = yield (0, cross_fetch_1.default)(urlObj, {
                    signal: controller.signal,
                });
                clearTimeout(id);
                if (!res.ok) {
                    throw new Error("unexpected response code: " + res.status);
                }
                const typeRes = validateFeatureFlagsResponse(yield res.json());
                if (!typeRes || !typeRes.success) {
                    throw new Error("unable to validate response");
                }
                flags = typeRes.flags;
                success = true;
            }
            catch (e) {
                console.error("[Bucket] error fetching flags: ", e);
            }
            finally {
                if (success) {
                    cache.set(url, { success, flags, attemptCount: 0 });
                }
                else {
                    const current = cache.get(url);
                    if (current) {
                        // if there is a previous failure cached, increase the attempt count
                        cache.set(url, {
                            success: current.success,
                            flags: current.flags,
                            attemptCount: current.attemptCount + 1,
                        });
                    }
                    else {
                        // otherwise cache if the request failed and there is no previous version to extend
                        // to avoid having the UI wait and spam the API
                        cache.set(url, { success: false, flags, attemptCount: 1 });
                    }
                }
                delete dedupeFetch[url];
            }
            return flags;
        });
        dedupeFetch[url] = fetchFlagsInner();
        return dedupeFetch[url];
    });
}
exports.fetchFlags = fetchFlags;
const FLAG_FETCH_DEFAULT_TIMEOUT_MS = 5000;
exports.FLAGS_STALE_MS = 60000; // turn stale after 60 seconds, optionally reevaluate in the background
exports.FLAGS_EXPIRE_MS = 7 * 24 * 60 * 60 * 1000; // expire entirely after 7 days
const FAILURE_RETRY_ATTEMPTS = 3;
const localStorageCacheKey = `__bucket_flags`;
const cache = new flags_cache_1.FlagCache({
    get: () => localStorage.getItem(localStorageCacheKey),
    set: (value) => localStorage.setItem(localStorageCacheKey, value),
    clear: () => localStorage.removeItem(localStorageCacheKey),
}, exports.FLAGS_STALE_MS, exports.FLAGS_EXPIRE_MS);
function clearCache() {
    cache.clear();
}
exports.clearCache = clearCache;
// fetch feature flags
function getFlags({ apiBaseUrl, context, staleWhileRevalidate = true, timeoutMs = FLAG_FETCH_DEFAULT_TIMEOUT_MS, cacheNegativeAttempts = FAILURE_RETRY_ATTEMPTS, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const flattenedContext = flattenJSON({ context });
        const params = new URLSearchParams(flattenedContext);
        // sort the params to ensure that the URL is the same for the same context
        params.sort();
        const url = `${apiBaseUrl}/features/enabled?` + params.toString();
        const cachedItem = cache.get(url);
        // if there's no cached item OR the cached item is a failure and we haven't retried
        // too many times yet - fetch now
        if (!cachedItem ||
            (!cachedItem.success &&
                (cacheNegativeAttempts === false ||
                    cachedItem.attemptCount < cacheNegativeAttempts))) {
            return fetchFlags(url, timeoutMs);
        }
        // cachedItem is a success or a failed attempt that we've retried too many times
        if (cachedItem.stale) {
            // serve successful stale cache if `staleWhileRevalidate` is enabled
            if (staleWhileRevalidate && cachedItem.success) {
                // re-fetch in the background, return last successful value
                fetchFlags(url, timeoutMs).catch(() => {
                    // we don't care about the result, we just want to re-fetch
                });
                return cachedItem.flags;
            }
            return fetchFlags(url, timeoutMs);
        }
        // serve cached items if not stale and not expired
        return cachedItem.flags;
    });
}
exports.getFlags = getFlags;
//# sourceMappingURL=flags-fetch.js.map