"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.closeAblySSEChannel = exports.openAblySSEChannel = exports.AblySSEChannel = void 0;
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const config_1 = require("./config");
const prompt_storage_1 = require("./prompt-storage");
const ABLY_TOKEN_ERROR_MIN = 40000;
const ABLY_TOKEN_ERROR_MAX = 49999;
class AblySSEChannel {
    constructor(userId, channel, ablyAuthUrl, sseHost, messageHandler, options) {
        var _a;
        this.userId = userId;
        this.channel = channel;
        this.ablyAuthUrl = ablyAuthUrl;
        this.sseHost = sseHost;
        this.messageHandler = messageHandler;
        this.isOpen = false;
        this.eventSource = null;
        this.retryInterval = null;
        this.debug = (_a = options === null || options === void 0 ? void 0 : options.debug) !== null && _a !== void 0 ? _a : false;
    }
    log(message, ...args) {
        if (this.debug) {
            console.log("[SSE]", message, ...args);
        }
    }
    warn(message, ...args) {
        console.warn("[SSE]", message, ...args);
    }
    err(message, ...args) {
        if (this.debug) {
            console.error("[SSE]", message, ...args);
        }
        throw new Error(message);
    }
    refreshTokenRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield (0, cross_fetch_1.default)(`${this.ablyAuthUrl}?userId=${encodeURIComponent(this.userId)}`, {
                method: "get",
                headers: {
                    "Content-Type": "application/json",
                    [config_1.SDK_VERSION_HEADER_NAME]: config_1.SDK_VERSION,
                },
            });
            if (res.ok) {
                const body = yield res.json();
                if (body.success) {
                    delete body.success;
                    const tokenRequest = body;
                    this.log("obtained new token request", tokenRequest);
                    return tokenRequest;
                }
            }
            this.err("server did not release a token request", res);
        });
    }
    refreshToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const cached = (0, prompt_storage_1.getAuthToken)(this.userId);
            if (cached && cached.channel === this.channel) {
                this.log("using existing token", cached.channel, cached.token);
                return cached.token;
            }
            const tokenRequest = yield this.refreshTokenRequest();
            const res = yield (0, cross_fetch_1.default)(`${this.sseHost}/keys/${encodeURIComponent(tokenRequest.keyName)}/requestToken`, {
                method: "post",
                headers: {
                    "Content-Type": "application/json",
                },
                body: JSON.stringify(tokenRequest),
            });
            if (res.ok) {
                const details = yield res.json();
                this.log("obtained new token", details);
                (0, prompt_storage_1.rememberAuthToken)(this.userId, this.channel, details.token, new Date(details.expires));
                return details.token;
            }
            this.err("server did not release a token", res);
        });
    }
    onError(e) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (e instanceof MessageEvent) {
                let errorCode;
                try {
                    const errorPayload = JSON.parse(e.data);
                    errorCode = (errorPayload === null || errorPayload === void 0 ? void 0 : errorPayload.code) && Number(errorPayload.code);
                }
                catch (error) {
                    this.warn("received unparseable error message", error, e);
                }
                if (errorCode &&
                    errorCode >= ABLY_TOKEN_ERROR_MIN &&
                    errorCode <= ABLY_TOKEN_ERROR_MAX) {
                    this.log("event source token expired, refresh required");
                    (0, prompt_storage_1.forgetAuthToken)(this.userId);
                }
            }
            else {
                const connectionState = (_a = e === null || e === void 0 ? void 0 : e.target) === null || _a === void 0 ? void 0 : _a.readyState;
                if (connectionState === 2) {
                    this.log("event source connection closed", e);
                }
                else if (connectionState === 1) {
                    this.warn("event source connection failed to open", e);
                }
                else {
                    this.warn("event source unexpected error occured", e);
                }
            }
            this.disconnect();
        });
    }
    onMessage(e) {
        let payload;
        try {
            if (e.data) {
                const message = JSON.parse(e.data);
                if (message.data) {
                    payload = JSON.parse(message.data);
                }
            }
        }
        catch (error) {
            this.warn("received unparseable message", error, e);
            return;
        }
        if (payload) {
            this.log("received message", payload);
            try {
                this.messageHandler(payload);
            }
            catch (error) {
                this.warn("failed to handle message", error, payload);
            }
            return;
        }
        this.warn("received invalid message", e);
    }
    onOpen(e) {
        this.log("event source connection opened", e);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isOpen) {
                this.warn("channel connection already open");
                return;
            }
            this.isOpen = true;
            try {
                const token = yield this.refreshToken();
                this.eventSource = new EventSource(`${this.sseHost}/sse?v=1.2&accessToken=${encodeURIComponent(token)}&channels=${encodeURIComponent(this.channel)}&rewind=1`);
                this.eventSource.addEventListener("error", (e) => this.onError(e));
                this.eventSource.addEventListener("open", (e) => this.onOpen(e));
                this.eventSource.addEventListener("message", (m) => this.onMessage(m));
                this.log("channel connection opened");
            }
            finally {
                this.isOpen = !!this.eventSource;
            }
        });
    }
    disconnect() {
        if (!this.isOpen) {
            this.warn("channel connection already closed");
            return;
        }
        if (this.eventSource) {
            this.eventSource.close();
            this.eventSource = null;
            this.log("channel connection closed");
        }
        this.isOpen = false;
    }
    open(options) {
        var _a, _b;
        const retryInterval = (_a = options === null || options === void 0 ? void 0 : options.retryInterval) !== null && _a !== void 0 ? _a : 1000 * 30;
        const retryCount = (_b = options === null || options === void 0 ? void 0 : options.retryCount) !== null && _b !== void 0 ? _b : 3;
        let retriesRemaining = retryCount;
        const tryConnect = () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.connect();
                retriesRemaining = retryCount;
            }
            catch (e) {
                if (retriesRemaining > 0) {
                    this.warn(`failed to connect, ${retriesRemaining} retries remaining`, e);
                }
                else {
                    this.warn(`failed to connect, no retries remaining`, e);
                }
            }
        });
        void tryConnect();
        this.retryInterval = setInterval(() => {
            if (!this.isOpen && this.retryInterval) {
                if (retriesRemaining <= 0) {
                    clearInterval(this.retryInterval);
                    this.retryInterval = null;
                    return;
                }
                retriesRemaining--;
                void tryConnect();
            }
        }, retryInterval);
    }
    close() {
        if (this.retryInterval) {
            clearInterval(this.retryInterval);
            this.retryInterval = null;
        }
        this.disconnect();
    }
    isActive() {
        return !!this.retryInterval;
    }
    isConnected() {
        return this.isOpen;
    }
}
exports.AblySSEChannel = AblySSEChannel;
function openAblySSEChannel(ablyAuthUrl, userId, channel, callback, options) {
    const sse = new AblySSEChannel(userId, channel, ablyAuthUrl, (options === null || options === void 0 ? void 0 : options.sseHost) || config_1.SSE_REALTIME_HOST, callback, { debug: options === null || options === void 0 ? void 0 : options.debug });
    sse.open();
    return sse;
}
exports.openAblySSEChannel = openAblySSEChannel;
function closeAblySSEChannel(channel) {
    channel.close();
}
exports.closeAblySSEChannel = closeAblySSEChannel;
//# sourceMappingURL=sse.js.map